/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    A layout widget that displays a column of input fields inside a container
 //    widget. The client specifies the type of input fields to displayed. There're
 //    several pre-defined input fields, like text and combo field. The client also
 //    can specify a custom field which the client must create before setting it to be
 //    managed by this layout container widget. Please see the specific public API of
 //    this widget to add pre-defined or custom input fields. 
 //
 //    The widget supports the number of visible input fields initially when the widget
 //    is created. If the client adds more then that number of visible input fields, the
 //    remaining fields are not displayed. The widget creates an action that allows the user 
 //    to view and to make visible of the hidden fields. Once a field is in display, the user
 //    cannot hide it again. Currently we don't have that capability built-in.
 // 
 //    The widget also supports action buttons to be created in the layout.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder");

dojo.require("dijit.Tooltip");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormEnumField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormCheckboxField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField");

(function() {

var ImageCache      = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var TextField       = com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField;
var EnumField       = com.ibm.asq.common.web.ui.internal.widgets.layout.FormEnumField;
var NumericField    = com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField;
var CheckboxField    = com.ibm.asq.common.web.ui.internal.widgets.layout.FormCheckboxField;
var FormToggleListField = com.ibm.asq.common.web.ui.internal.widgets.layout.FormToggleListField;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" dojoAttachEvent="onclick:_onBlur,onkeypress:_onKeyPressed" class="com-ibm-asq-common-widget-attribute-builder">' +
                    '</div>',
    
    // Summary:
    //    Constructs an instance of an AttributeBuilder object. It supports
    //    the following mixin attributes:
    //       var attrBuilder = new com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder({
    //          title: "Attribute Builder",  // provide a name for the builder panel.
    //          style: "panel", or "TBD",    // the style for the builder's panel, default is a collapsible section
    //          attributes: [{title: "Owner", name: "db.owner", type: AttributeBuilder.TEXT_FIELD, defaultValue: null},
    //                       {title: "Type", name: "db.type", type: AttributeBuilder.ENUM_FIELD, defaultValue: null, param: ["Type A","Taype B","Type C"]}
    //                      ],
    //       });      
    //  
    constructor: function()
    {
        this.title = null;
        this.summaryText = "";
        this.style = "panel";
        this.attributes = null;
        this.visibleAttributeCount = -1;
        this.labelWidth = 35;
        this.fieldAreaWidth = 58;
		this.initialOpen = true;
    },
     
    postCreate: function() 
    {
        this.attributeFields = [];
        this.visibleCount = 0;
        this.nonVisibleFields = [];
        this.actions = {};

        if (this.style == "panel") {
            var holder = document.createElement("div");
            this.domNode.appendChild(holder);
            this.sectionNode = new com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection({
                titleSize: "11pt",
                open: this.initialOpen
            }, holder);
            
            this.contentNode = document.createElement("div");
            this.sectionNode.containerNode.appendChild(this.contentNode);
            
            if (this.title) 
                this.setTitle(this.title);
            
            if(!this.summaryText)
                this.summaryText = "";
                
            this.sectionNode.setTitleMode(false);
            this.sectionNode.setAttributeAreaHidden(true);
            this.sectionNode.setSummaryAreaHidden(false);
            this.summaryNode = this.sectionNode.summaryNode;
            this.summaryNode.innerHTML = this.summaryText;
        }
        else {
            this.contentNode = document.createElement("div");
            this.domNode.appendChild(this.contentNode);            
        }
        
        this.containerNode = document.createElement("div");
        dojo.addClass(this.containerNode, "attribute-container");
        
        this.containerNode.style.width = this.fieldAreaWidth + "%";
        this.contentNode.appendChild(this.containerNode);

        this.actionNode = document.createElement("div");
        dojo.addClass(this.actionNode, "action-container");
        this.contentNode.appendChild(this.actionNode);
        
        // if attributes were specified in constructor time, create them.
        if(this.attributes && dojo.isArray(this.attributes)) {
            for(var i = 0; i < this.attributes.length; i++) {
                var attr = this.attributes[i];
                this.addAttribute(attr.name, attr.title, attr.type, attr.defaultValue, attr.param);
            }
        }
    },
    
    // Summary:
    //    Set the title of the pane.
    //
    setTitle: function(/*String*/ title)
    {
        if(this.sectionNode)
            this.sectionNode.setTitle(title);
    },

    // Summary:
    //    Adds a new pre-defined type input field to this layout widget.
    //
    addAttribute: function(name, title, promptTitle, type, defaultValue, params)
    {
        var field = null;
        var fieldArgs = {
            fieldLabel: title,
            labelWidthPercent: this.labelWidth
        };
        
        switch(type) {
            case AttributeBuilder.FIELD_TEXT:
                if(params) {
                    for(var key in params) {
                        fieldArgs[key] = params[key];
                    }
                }
                field = new TextField(fieldArgs);
                break;
            case AttributeBuilder.FIELD_ENUM:
                field = new EnumField(fieldArgs);
                if(params)
                    field.setEnumChoices(params);
                break;
            case AttributeBuilder.FIELD_NUMERIC:
                if(params) {
                    for(var key in params) {
                        fieldArgs[key] = params[key];
                    }
                }
                field = new NumericField(fieldArgs);
                break;
            case AttributeBuilder.FIELD_CHECKBOX:
                if(params && dojo.isString(params))
                    fieldArgs['checkboxText'] = params;
                field = new CheckboxField(fieldArgs);
                break;                
            case AttributeBuilder.FIELD_TOGGLELIST:
                if(params) {
                    for(var key in params) {
                        fieldArgs[key] = params[key];
                    }
                }
                field = new FormToggleListField(fieldArgs);
                break;
            default:
                throw new Error("Specified type is not one of the supported attribute type: " + type);
        }
        if(defaultValue) {
            field.setValue(defaultValue);
            field.defaultValue = defaultValue;
        }
        this._addField(name, title, promptTitle, field);            
    },
    
    // Summary:
    //    Adds a new custom type input field to this layout widget. The created
    //    field widget must be a subclass of the AbstractFormField widget.
    //
    addCustomAttribute: function(name, title, promptTitle, field)
    {
        field.setLabelWidthPercentage(this.labelWidth);
        field.setLabel(title);
        
        this._addField(name, title, promptTitle, field);
    },
    
    // Summary:
    //    Retrieves the values from the visible input fields. Fields that are not
    //    visible will not have values returned. An associative array is returned
    //    which is indexed with the input field's id name to the input values.
    //  
    getAttributeValues: function()
    {
        var values = {};
        
        for(var name in this.attributeFields) {
            var field = this.attributeFields[name];
            values[name] = field.getValue();
        }
        return values;
    },
    
    // Summary:
    //    Returns whether there is error in the attribute input fields. 
    //
    hasError: function()
    {
    },
    
    // Summary:
    //    Add actions to the builder's panel. 
    //
    addAction: function(name, title, callback, callbackData, isDefault)
    {
        var button = document.createElement("button");
        button.innerHTML = title;
        button.name = name;
        
        // add action's parameters inside our widget. We'll
        // look it up when the button is activated.
        //
        this.actions[name] = {'name': name, 'callback': callback, 'callbackData': callbackData};
        this.actionNode.appendChild(button);
        
        if(isDefault) {
            this.defaultAction = name;
        }
        this.connect(button, "onclick", "_buttonActivated");
    },
    
    _addField: function(name, title, promptTitle, field)
    {
        // if we're not showing all of the attributes at once,
        // and we've reached the limit of visible attributes, save
        // the attribute field to the hidden list.
        if(this.visibleAttributeCount > 0 && this.visibleCount == this.visibleAttributeCount) {
            this.nonVisibleFields[name] = {'name':name, 'title': promptTitle, 'field': field};
            
            // show the add hidden action to allow the user
            // to view and select them.
            this._addSelectHiddenAttributeAction();
            return;
        }
        this.visibleCount++;
        this.attributeFields[name] = field;
        var attr = document.createElement("div");
        attr.className = "attribute-display";
        attr.appendChild(field.domNode);
        this.containerNode.appendChild(attr);        
    },

    _buttonActivated: function(event)
    {
        var evt = event || window.event;
        if(evt.target) {
            var action = this.actions[evt.target.name];
            
            if(action) {
                action.callback(action.name, action.callbackData);
            }
        }
    },
    
    _addSelectHiddenAttributeAction: function()
    {
        // we've already created the show hidden action. Don't
        // have to do anything.
        if(this.showHiddenAction) {
            return;
        }
        
        // create a new span to show the summary and the new
        // action.
        this.summaryNode.innerHTML = "";
        var promptArea = document.createElement("span");
        var promptNode = document.createElement("span");
        
        promptNode.innerHTML = this.summaryText;
        promptArea.appendChild(promptNode);

        var args = {
            'showLabel': false,
            'class': "com-ibm-asq-common-widget-attribute-builder-action",
            'label': MessageUtil.getMessage(MessageBundle.COMMON, "add_attribute"),
            'iconClass': "add-attr-icon-image",
            'onClick': dojo.hitch(this, "_openAttributePicker")
        };
        this.showHiddenAction = new dijit.form.Button(args);
        promptArea.appendChild(this.showHiddenAction.domNode);
        this.summaryNode.appendChild(promptArea);
    },
   
    _getHiddenAttributes: function()
    {
        var list = [];
        
        // returns the current list of hidden attributes in a
        // format that can be used to create options to represent
        // them: name, value pair. The name is the visible title 
        // and the value is the id for it.
        for(var name in this.nonVisibleFields) {
            var attr = this.nonVisibleFields[name];
            list.push({name:attr.title, value:attr.name});
        }
        return list;
    },
    
    _openAttributePicker: function()
    {
		// if the user click on the add attribute and the section
		// is not open, open the section first.
		if(this.sectionNode && !this.sectionNode.open) {
			this.sectionNode.toggle();
		}
        if(this._opened)
            return;

        this.attrPicker = new com.ibm.asq.common.web.ui.internal.widgets.layout.AttributePicker({
            'promptText': "",
            'attributeItems': this._getHiddenAttributes(),
            'onClose': dojo.hitch(this, "_closeAttributePicker"),
            'onAdd': dojo.hitch(this, "_addHiddenAttributes")
        });

        // popup will register this widget with the focus manager.
        // By having a _onBlur method in this widget, whenever a click happens outside
        // of this widget, that method will get called.
        dijit.popup.open({
            parent: this,
            popup: this.attrPicker,
            around: this.showHiddenAction.domNode
        });         
        this._opened = true;       
        
    } ,
    
    _closeAttributePicker: function()
    {
        if(this._opened){
            dijit.popup.close(this.attrPicker);
            this._opened = false;
        }           
    },

    _addHiddenAttributes: function(attributes)
    {
        // allow the new number of visible items.
        this.visibleAttributeCount += attributes.length;
        
        // add the selected hidden attribute to the visible
        // list and delete the newly added ones from the hidden
        // list.
        for(var i = 0; i < attributes.length; i++) {
            var name = attributes[i];
            var attr = this.nonVisibleFields[name];
            this._addField(attr.name, attr.title, null, attr.field);
        }
        
        for(var i = 0; i < attributes.length; i++) {
            delete this.nonVisibleFields[attributes[i]];
        }
        
        // refresh the hidden list, now that we've added
        // more to visible.
        var hiddenList = this._getHiddenAttributes();
        this.attrPicker.attributeItems = hiddenList;
        this.attrPicker.refreshList();
        
        this.showHiddenAction.setDisabled(hiddenList.length == 0);
    },
    
    // summary: 
    //    Called by the focus manager in focus.js when focus moves outside of the
    //    AttributeBuilder widget (or it's descendants).
    //
    _onBlur: function()
    {
        this._closeAttributePicker();
    },
    
    _onKeyPressed: function(event)
    {
        if (this.defaultAction) {
            var evt = event || window.event;
            var c = (evt.keyCode || 0);
            
            if (c == dojo.keys.ENTER) {
                var action = this.actions[this.defaultAction];
                if(action) {
                    action.callback(action.name, action.callbackData);
                }
            }
        }
    }
});

var AttributeBuilder = com.ibm.asq.common.web.ui.internal.widgets.layout.AttributeBuilder;

AttributeBuilder.FIELD_TEXT = "TextField";
AttributeBuilder.FIELD_ENUM = "EnumField";
AttributeBuilder.FIELD_NUMERIC = "NumericField";
AttributeBuilder.FIELD_CHECKBOX = "CheckboxField";
AttributeBuilder.FIELD_TOGGLELIST = "CheckboxListField";

})();
