/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //   The TableCellRenderer is an "interface" object that the TableViewer uses
 //   to draw a cell in the table. Specific cell renderer must be subclassed from this
 //   interface class and implements all public methods defined in this class. The
 //   cell renderer, employed by the TableViewer, is responsible for drawing a cell in 
 //   a table row. Specific subclasses will render the cell data in the intended display
 //   style. For example, a basic text cell renderer will draw the text data in the cell,
 //   or a image cell renderer will draw an image that represents the data in the cell, etc.
 //   More complex cell renderer could allow the user to change the value of the cell
 //   data. It's the responsibility of the renderer to send the value changed event to
 //   the supplied cell modifier.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");

(function() {

var WordWrappingUtil = com.ibm.asq.common.web.ui.internal.util.wordWrapping;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer", null,
{    
    // Summary:
    //    Renders the cell content. The specific subclass will draw the appropriate
    //    widget for the cell rendition. 
    //    @param viewer the viewer that uses this cell renderer
    //    @param cellData The data to be displayed. The data can be an object to hold sufficient data for 
    //    the appropriate renderer type (obtained from the table's content provider)
    //    @param extendedDisplayInfo Additional display info about the cell.
    //    @param cellModifier The modifier object that can receive value changed events from the cell.
    //    @param rowObject The data object for the row. It's the client data that represent the row
    //    @param columnIndex  the column index of the cell
    //    @param cellWidthHint optional parameter to indicate the width of the cell, which the cell content *should not* excess
    //    @return the domNode for the cell content.
    //
    renderCell: function(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, /*optional*/cellWidthHint)
    {
        // Subclass must provide a dom node to display the cell data.
        // In addition, if the cellWidthHint parameter is provided, the cell
        // renderer should render the content within that width boundry. Use
        // this base class's _clipText method for clipping the textual contents.
        //
        return null;
    },
    
    // Summary:
    //   Attaches a tooltip to the specified node. If the specified tooltipText is null,
    //   or is empty, no tooltip will be created for the node.
    //
    _attachTooltipToCell: function(node, tooltipText)
    {
        var tooltip = null;
        
        if(tooltipText && tooltipText.length > 0)
            tooltip = new net.jazz.ajax.ui.internal.HoverPopup({label: tooltipText, connectId:[node]});
        return tooltip;
    },
    
    // Summary:
    //    Clips the text at the specified width. The text will be clipped and an ellipsis is
    //    added to the end of the text.
    //
    ClipTextClassName: "clip-cell-nowrap",
    _clipText: function(viewer, text, maxWidth)
    {
        return WordWrappingUtil.clipText(text, this.ClipTextClassName, maxWidth, viewer.ruler, true, /*threshold*/dojo.isIE ? 16 : 12);
    },
    
    // Summary:
    //    Class method to handle compact cell mode in the table viewer. All subclasses should use
    //    this method to configure a textual display node to handle text clipping and the proper
    //    style sheet settings for compact cell.
    //
    _handleCompactCellText: function(viewer, text, extendedDisplayInfo, node, maxWidth)
    {
        var val = this._clipText(viewer, text, maxWidth);
        
        if(val.clipped) {
			var ttText = text;
            if (extendedDisplayInfo && extendedDisplayInfo['tooltip']) {
                ttText = text + "<br></br>" + extendedDisplayInfo['tooltip'];
            }
            node.tooltip = this._attachTooltipToCell(node, ttText);   
        }
        dojo.addClass(node, this.ClipTextClassName);
        
        var ret = (dojo.isString(val.text) && val.text.length > 0) ? val.text : " ";
        return ret;
    },
    
    // Summary:
    //    Class method to render text in a table cell. When possible, always use this method
    //    to draw text in a table cell. This method properly adjusts the text being displayed
    //    to conform to the table's compact settings and preserves the client's character in the
    //    display. This method expects a dom node (at least a 'span') for the display node.
    //
    _renderCellText: function(viewer, cellData, extendedDisplayInfo, node, cellWidthHint)
    {
        var text = (cellData == null || (dojo.isString(cellData) && cellData.length == 0)) ? "" : cellData + "";
        
        if(viewer.isCompactMode && typeof cellWidthHint != "undefined") {
            text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, node, cellWidthHint);
        }
        else {
            if(text.length == 0)
                text = " ";
            
            if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) 
                this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);
        }
        
        // wrap the text with a text node to ensure all characters are displayed. 
        // If not, HTML tag characters like '<' could be escaped.
        var textNode = document.createTextNode(text);
        node.appendChild(textNode);
    }
    
});

})();
